/****************************************************************************
**
** Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of the documentation of Qt. It was originally
** published as part of Qt Quarterly.
**
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file.  Please review the following information
** to ensure GNU General Public Licensing requirements will be met:
** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
** exception, Nokia gives you certain additional rights. These rights
** are described in the Nokia Qt GPL Exception version 1.3, included in
** the file GPL_EXCEPTION.txt in this package.
**
** Qt for Windows(R) Licensees
** As a special exception, Nokia, as the sole copyright holder for Qt
** Designer, grants users of the Qt/Eclipse Integration plug-in the
** right for the Qt/Eclipse Integration to link to functionality
** provided by Qt Designer and its related libraries.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
**
****************************************************************************/

#include <QtGui>
#include "imagewrapper.h"

/**
    \class ImageWrapper

    \brief This class provides a wrapper or proxy for an image in C++ that
    can be manipulated from Qt Script.

    Instances of this class are not intended to be instantiated from
    Qt Script, but instead created in C++ and passed to scripts for
    processing.
*/

ImageWrapper::ImageWrapper(QObject *parent)
    : QObject(parent)
{
}

void ImageWrapper::setImage(const QImage &newImage)
{
    image = newImage;
    original = newImage;
}

/**
    Returns the width of the image in pixels.

    \see height()
*/

int ImageWrapper::width() const
{
    return image.width();
}

/**
    Returns the height of the image in pixels.

    \see width()
*/

int ImageWrapper::height() const
{
    return image.height();
}

/**
    Resets the image to its original state.
*/

void ImageWrapper::reset()
{
    image = original;
}

/**
    Scales the image by the horizontal and vertical factors specified by
    \a xScale and \a yScale respectively.

    If \a publish is true, the imageChanged() signal will be emitted to
    notify other components that the image has been changed.
*/

void ImageWrapper::scale(qreal xScale, qreal yScale, bool publish)
{
    image = image.scaled(int(xScale*image.width()), int(yScale*image.height()));
    if (publish)
        emit imageChanged(image);
}

/**
    Blurs the image by an arbitrary amount.

    If \a publish is true, the imageChanged() signal will be emitted to
    notify other components that the image has been changed.
*/

void ImageWrapper::blur(bool publish)
{
    QImage newImage(image.width() + 16, image.height(),
                    QImage::Format_ARGB32);
    for (int row = 0; row < image.height(); ++row) {
        for (int column = 0; column < newImage.width(); ++column) {
            qreal red = 0;
            qreal green = 0;
            qreal blue = 0;
	    qreal alpha = 0;
            qreal total = 0;
            int sourceColumn = column - 8;
            for (int i = qBound(0, sourceColumn-8, image.width());
                     i < qBound(0, sourceColumn+9, image.width()); ++i) {
                QRgb pixel = image.pixel(i, row);
                red += qRed(pixel);
                green += qGreen(pixel);
                blue += qBlue(pixel);
		alpha += qAlpha(pixel);
                total++;
            }
            if (total > 0) {
                red /= total;
                green /= total;
                blue /= total;
		alpha /= total;
                newImage.setPixel(column, row,
                    qRgba(int(red), int(green), int(blue), int(alpha)));
            } else
                newImage.setPixel(column, row, qRgba(0, 0, 0, 0));
        }
    }
    image = newImage;
    if (publish)
        emit imageChanged(image);
}

/**
    Converts the image to a grayscale representation.

    If \a publish is true, the imageChanged() signal will be emitted to
    notify other components that the image has been changed.
*/

void ImageWrapper::grayscale(bool publish)
{
    QImage newImage(image.width(), image.height(),
                    QImage::Format_ARGB32);
    for (int row = 0; row < image.height(); ++row) {
        for (int column = 0; column < image.width(); ++column) {
            QRgb pixel = image.pixel(column, row);
            int gray = qGray(pixel);
            newImage.setPixel(column, row, qRgba(gray, gray, gray, qAlpha(pixel)));
        }
    }
    image = newImage;
    if (publish)
        emit imageChanged(image);
}

/**
    Adds a label to the image with the given \a text at the position specified
    by the respective \a x and \a y coordinates. The font name and size are
    given by \a fontName and \a size.

    If \a publish is true, the imageChanged() signal will be emitted to
    notify other components that the image has been changed.
*/

void ImageWrapper::label(int x, int y, const QString &fontName, int size, const QString &text, bool publish)
{
    QPainter painter;
    painter.begin(&image);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
    QFont font(fontName);
    font.setPixelSize(size);
    painter.setFont(font);
    QFontMetricsF fontMetrics(font, &image);
    QRectF rect = fontMetrics.boundingRect(text);
    rect.translate(x - rect.x() + 4, y - rect.y() + 4);
    painter.setPen(Qt::NoPen);
    painter.setBrush(QColor(255, 255, 255, 160));
    painter.drawRoundRect(rect.adjusted(-4, -4, 4, 4), 10, 10);
    painter.setPen(QColor(0, 0, 0, 255));
    painter.drawText(rect, Qt::AlignCenter, text);
    painter.end();
    if (publish)
        emit imageChanged(image);
}

/**
    Transforms the image to create a mask by converting all non-white pixels
    to black pixels.

    If \a publish is true, the imageChanged() signal will be emitted to
    notify other components that the image has been changed.
*/

void ImageWrapper::mask(bool publish)
{
    QImage newImage(image.width(), image.height(),
                    QImage::Format_ARGB32);
    for (int row = 0; row < image.height(); ++row) {
        for (int column = 0; column < image.width(); ++column) {
            QRgb pixel = image.pixel(column, row);
            int gray = qGray(pixel);
            if (gray < 255) gray = 0; else gray = 255;
            newImage.setPixel(column, row, qRgba(gray, gray, gray, qAlpha(pixel)));
        }
    }
    image = newImage;
    if (publish)
        emit imageChanged(image);
}

/**
    Creates a shadow for the image based on the areas that are not transparent
    and superimposes the image onto it at a position relative to the top-left
    corner. The size of the shadow is 

    The horizontal and vertical displacements of the image relative to
    the shadow underneath are given by \a xOffset and \a yOffset.

    The size of the shadow and the resulting image is wider and taller than
    the source image by \a extent pixels. This parameter can be used to
    ensure that there is enough space to show the displaced image over its
    shadow.

    If \a publish is true, the imageChanged() signal will be emitted to
    notify other components that the image has been changed.
*/

void ImageWrapper::shadow(int extent, int xOffset, int yOffset,
                          bool publish)
{
    QImage oldImage = image;
    QImage newImage(image.width() + extent, image.height() + extent,
                    QImage::Format_ARGB32);

    newImage.fill(qRgba(0, 0, 0, 0));

    for (int row = 0; row < image.height(); ++row) {
        for (int column = 0; column < newImage.width(); ++column) {
            int red = 0;
            int green = 0;
            int blue = 0;
	    int alpha = 0;
            int total = 0;
            int sourceColumn = column - extent/2;
            for (int i = qBound(0, sourceColumn - extent/2, image.width());
                     i < qBound(0, sourceColumn + extent/2 + 1, image.width()); ++i) {
                QRgb pixel = image.pixel(i, row);
                red += qRed(pixel);
                green += qGreen(pixel);
                blue += qBlue(pixel);
		alpha += qAlpha(pixel);
                total++;
            }
            if (total > 0) {
                red /= total;
                green /= total;
                blue /= total;
		alpha /= total;
                newImage.setPixel(column, row + extent/2,
                                  qRgba(red, green, blue, alpha));
            }
        }
    }

    image = newImage;
    mask(false);
    int dx = (image.width() - oldImage.width()) / 2;
    int dy = (image.height() - oldImage.height()) / 2;

    QPainter painter;
    painter.begin(&image);
    painter.drawImage(dx + xOffset, dy + yOffset, oldImage);
    painter.end();

    if (publish)
        emit imageChanged(image);
}
