/****************************************************************************
**
** Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of the documentation of Qt. It was originally
** published as part of Qt Quarterly.
**
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file.  Please review the following information
** to ensure GNU General Public Licensing requirements will be met:
** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
** exception, Nokia gives you certain additional rights. These rights
** are described in the Nokia Qt GPL Exception version 1.3, included in
** the file GPL_EXCEPTION.txt in this package.
**
** Qt for Windows(R) Licensees
** As a special exception, Nokia, as the sole copyright holder for Qt
** Designer, grants users of the Qt/Eclipse Integration plug-in the
** right for the Qt/Eclipse Integration to link to functionality
** provided by Qt Designer and its related libraries.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
**
****************************************************************************/

#include <QtGui>
#include <QtHelp>
#include "helpbrowser.h"
#include "highlighter.h"
#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    QMenu *fileMenu = menuBar()->addMenu(tr("&File"));
    QAction *openAction = fileMenu->addAction(tr("&Open..."));
    openAction->setShortcut(QKeySequence::Open);
    QAction *exitAction = fileMenu->addAction(tr("E&xit"));
    exitAction->setShortcut(QKeySequence(tr("Ctrl+Q")));

    QMenu *scriptMenu = menuBar()->addMenu(tr("&Script"));
    QAction *openScriptAction = scriptMenu->addAction(tr("&Open Script Window"));
    openScriptAction->setEnabled(false);
    QAction *runScriptAction = scriptMenu->addAction(tr("&Run Script"));

    QMenu *helpMenu = menuBar()->addMenu(tr("&Help"));
    QAction *openHelpAction = helpMenu->addAction(tr("&Open Help Window"));
    openHelpAction->setEnabled(true);

    scriptWindow = new QDockWidget(tr("Script"), this);
    scriptEditor = new QTextEdit;
    scriptWindow->setWidget(scriptEditor);
    addDockWidget(Qt::BottomDockWidgetArea, scriptWindow);

    QHelpEngine *helpEngine = new QHelpEngine(DOCS_PATH, this);
    helpEngine->setupData();

    helpWindow = new QDockWidget(tr("Help"), this);
    QSplitter *helpPanel = new QSplitter(Qt::Horizontal);
    HelpBrowser *helpBrowser = new HelpBrowser(helpEngine);

    helpPanel->insertWidget(0, helpEngine->contentWidget());
    helpPanel->insertWidget(1, helpBrowser);
    helpPanel->setStretchFactor(1, 1);
    helpWindow->setWidget(helpPanel);
    addDockWidget(Qt::TopDockWidgetArea, helpWindow);

    QFile defaultJs(":/resources/default.js");
    defaultJs.open(QFile::ReadOnly);
    scriptEditor->setPlainText(defaultJs.readAll());
    defaultJs.close();
    new Highlighter(scriptEditor->document());
    
    image = new ImageWrapper(this);
    QScriptValue imageObject = engine.newQObject(image);
    engine.globalObject().setProperty("image", imageObject);

    connect(openAction, SIGNAL(triggered()), this, SLOT(openFile()));
    connect(exitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
    connect(openHelpAction, SIGNAL(triggered()), helpWindow, SLOT(show()));
    connect(openScriptAction, SIGNAL(triggered()), scriptWindow, SLOT(show()));
    connect(openHelpAction, SIGNAL(triggered()), helpWindow, SLOT(raise()));
    connect(openScriptAction, SIGNAL(triggered()), scriptWindow, SLOT(raise()));
    connect(runScriptAction, SIGNAL(triggered()), this, SLOT(runScript()));
    
    connect(image, SIGNAL(imageChanged(const QImage &)),
            this, SLOT(updateImage(const QImage &)));

    connect(helpEngine->contentWidget(), SIGNAL(linkActivated(const QUrl &)),
            helpBrowser, SLOT(setSource(const QUrl &)));

    connect(helpWindow, SIGNAL(visibilityChanged(bool)),
            openHelpAction, SLOT(setDisabled(bool)));
    connect(scriptWindow, SIGNAL(visibilityChanged(bool)),
            openScriptAction, SLOT(setDisabled(bool)));

    imageList = new QListWidget;
    imageList->setViewMode(QListView::IconMode);
    setCentralWidget(imageList);
    setWindowTitle(tr("Image Processor"));

    QTimer::singleShot(0, this, SLOT(setup()));
}

void MainWindow::setup()
{
    QImage newImage;
    newImage.load(":/resources/image.png");
    newImage = newImage.convertToFormat(QImage::Format_ARGB32);
    image->setImage(newImage);
    int size = qMin(128, qMax(newImage.width(), newImage.height()));
    imageList->setIconSize(QSize(size, size));
    updateImage(newImage);
}

void MainWindow::openFile()
{
    QStringList imageFormats;
    foreach (QByteArray format, QImageReader::supportedImageFormats())
        imageFormats.append(QString("*.") + format);

    QString path = QFileDialog::getOpenFileName(this, tr("Open File"),
        currentPath, tr("Image files (%1)").arg(imageFormats.join(" ")));

    if (path.isEmpty())
        return;

    QImage newImage;
    if (newImage.load(path)) {
        imageList->clear();
        newImage = newImage.convertToFormat(QImage::Format_ARGB32);
        image->setImage(newImage);
        int size = qMin(128, qMax(newImage.width(), newImage.height()));
        imageList->setIconSize(QSize(size, size));
        updateImage(newImage);
        currentPath = path;
    }
}

void MainWindow::updateImage(const QImage &newImage)
{
    QListWidgetItem *listItem = new QListWidgetItem(imageList);
    listItem->setIcon(QIcon(QPixmap::fromImage(newImage)));
}

void MainWindow::runScript()
{
    QListWidgetItem *originalItem = imageList->takeItem(0);
    imageList->clear();
    imageList->addItem(originalItem);
    image->reset();

    QScriptValue result = engine.evaluate(scriptEditor->toPlainText());
    if (result.isError())
        QMessageBox::warning(this, tr("Error from QtScript"),
                             result.toString());
}
